package io.tswf.minecraft.blockventure.modloadercpw

import com.google.auto.service.AutoService
import cpw.mods.jarhandling.SecureJar
import cpw.mods.modlauncher.api.IEnvironment
import cpw.mods.modlauncher.api.IModuleLayerManager
import cpw.mods.modlauncher.api.ITransformationService
import cpw.mods.modlauncher.api.ITransformer
import cpw.mods.modlauncher.api.IncompatibleEnvironmentException
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.modloader.scanner.ModsScanner

@AutoService(ITransformationService)
@Slf4j
@CompileStatic
class BlockventureModsDiscoverTransformationService implements ITransformationService {

    @Override
    String name() {
        return this.class.name
    }

    @Override
    void initialize(IEnvironment environment) {
        log.info("${this.class.simpleName}.initialize")
    }

    @Override
    void onLoad(IEnvironment env, Set<String> otherServices) throws IncompatibleEnvironmentException {
        log.info("${this.class.simpleName}.initialize")
    }

    @Override
    List<Resource> completeScan(IModuleLayerManager layerManager) {
        log.info("${this.class.simpleName}.completeScan")
        def result = new ArrayList<Resource>()

        def prevClassloader = ClassLoader.contextClassloader()
        try {
            Thread.currentThread().setContextClassLoader(this.class.classLoader)

            log.info("Temporary set classloader to ${ClassLoader.contextClassloader()}")

            def foundBlockventureMods = ModsScanner.scanMods()

            if (foundBlockventureMods.isNotNullOrEmpty()) {
                def modFiles = foundBlockventureMods.map { it.runtimeMeta.fileFromRead }
                def modFilesResources = modFiles
                    .map { SecureJar.from(it.toPath()) }

                log.info("Appending found blockventure mods to classloading ${IModuleLayerManager.Layer.GAME.name()} cpw modlauncher layer: ${modFiles}")

                result << new Resource(IModuleLayerManager.Layer.GAME, modFilesResources)
            } else {
                log.info("No one blockventure mod found, nothing to append to cpw mod launcher. Skipping...")
            }

        } finally {
            Thread.currentThread().setContextClassLoader(prevClassloader)
        }

        return result
    }

    @Override
    List<Resource> beginScanning(IEnvironment environment) {
        log.info("${this.class.simpleName}.beginScanning")
        def result = new ArrayList<Resource>()

        def prevClassloader = ClassLoader.contextClassloader()
        try {
            Thread.currentThread().setContextClassLoader(this.class.classLoader)

            log.info("Temporary set classloader to ${ClassLoader.contextClassloader()}")
            log.info("Classpath prop: ${System.getProperty("class.path")}, list: ${System.classpathList}")

            System.classpathList.stream()
                    .filter { it.protocol == "file" }
                    .map { new File(it.path) }
                    .filter {file ->
                        def fileTree = file.asFileTree()
                        log.info("File tree is ${fileTree}")
                        return fileTree.doThenClose {
                            def success = it.getEntry("io/tswf/minecraft/blockventure/modloader/scanner/ModsScanner.class") != null
                            if (!success) {
                                log.error("Therer is no entry in ${file.absoluteCanonicalFile.path}!")
                            }

                            return success
                        }
                    }
                    .filter { it != null }
                    .firstOrNull()
                    ?.let {
                        log.info("Found mod scanner core dependency ${it.absoluteCanonicalFile.path}, injecting into cpw mod launcher...")
                        result << new Resource(IModuleLayerManager.Layer.SERVICE, [ SecureJar.from(it.toPath()) ])
                    }

        } finally {
            Thread.currentThread().setContextClassLoader(prevClassloader)
        }

        return result
    }

    @Override
    List<ITransformer> transformers() {
        log.info("${this.class.simpleName}.transformers")
        return []
    }
}
